@@ -19,6 +19,7 @@ import stat
 import time
 import platform, subprocess, operator, os, shutil, re
 import collections
+from mesonbuild import mlog
 
 from glob import glob
 
@@ -35,12 +36,11 @@ def detect_meson_py_location():
         # $ <mesontool> <args> (gets run from /usr/bin/<mesontool>)
         in_path_exe = shutil.which(c_fname)
         if in_path_exe:
-            m_dir, c_fname = os.path.split(in_path_exe)
-            # Special case: when run like "./meson.py <opts>",
-            # we need to expand it out, because, for example,
-            # "ninja test" will be run from a different directory.
-            if m_dir == '.':
+            if not os.path.isabs(in_path_exe):
                 m_dir = os.getcwd()
+                c_fname = in_path_exe
+            else:
+                m_dir, c_fname = os.path.split(in_path_exe)
     else:
         m_dir = os.path.abspath(c_dir)
 
@@ -76,6 +76,34 @@ else:
     python_command = [sys.executable]
     meson_command = python_command + [detect_meson_py_location()]
 
+def is_ascii_string(astring):
+    try:
+        if isinstance(astring, str):
+            astring.encode('ascii')
+        if isinstance(astring, bytes):
+            astring.decode('ascii')
+    except UnicodeDecodeError:
+        return False
+    return True
+
+def check_direntry_issues(direntry_array):
+    import locale
+    # Warn if the locale is not UTF-8. This can cause various unfixable issues
+    # such as os.stat not being able to decode filenames with unicode in them.
+    # There is no way to reset both the preferred encoding and the filesystem
+    # encoding, so we can just warn about it.
+    e = locale.getpreferredencoding()
+    if e.upper() != 'UTF-8' and not is_windows():
+        if not isinstance(direntry_array, list):
+            direntry_array = [direntry_array]
+        for de in direntry_array:
+            if is_ascii_string(de):
+                continue
+            mlog.warning('''You are using {!r} which is not a Unicode-compatible '
+locale but you are trying to access a file system entry called {!r} which is
+not pure ASCII. This may cause problems.
+'''.format(e, de), file=sys.stderr)
+
 # Put this in objects that should not get dumped to pickle files
 # by accident.
 import threading
@@ -259,6 +287,9 @@ def is_osx():
 def is_linux():
     return platform.system().lower() == 'linux'
 
+def is_android():
+    return platform.system().lower() == 'android'
+
 def is_haiku():
     return platform.system().lower() == 'haiku'
 
@@ -273,6 +304,12 @@ def is_cygwin():
 def is_debianlike():
     return os.path.isfile('/etc/debian_version')
 
+def is_dragonflybsd():
+    return platform.system().lower() == 'dragonfly'
+
+def is_freebsd():
+    return platform.system().lower() == 'freebsd'
+
 def for_windows(is_cross, env):
     """
     Host machine is windows?
@@ -321,6 +358,18 @@ def for_darwin(is_cross, env):
         return env.cross_info.config['host_machine']['system'] == 'darwin'
     return False
 
+def for_android(is_cross, env):
+    """
+    Host machine is Android?
+
+    Note: 'host' is the machine on which compiled binaries will run
+    """
+    if not is_cross:
+        return is_android()
+    elif env.cross_info.has_host():
+        return env.cross_info.config['host_machine']['system'] == 'android'
+    return False
+
 def for_haiku(is_cross, env):
     """
     Host machine is Haiku?
@@ -547,7 +596,8 @@ def do_conf_file(src, dst, confdata):
     return missing_variables
 
 def dump_conf_header(ofilename, cdata):
-    with open(ofilename, 'w', encoding='utf-8') as ofile:
+    ofilename_tmp = ofilename + '~'
+    with open(ofilename_tmp, 'w', encoding='utf-8') as ofile:
         ofile.write('''/*
  * Autogenerated by the Meson build system.
  * Do not edit, your changes will be lost.
@@ -569,6 +619,7 @@ def dump_conf_header(ofilename, cdata):
                 ofile.write('#define %s %s\n\n' % (k, v))
             else:
                 raise MesonException('Unknown data type in configuration file entry: ' + k)
+    replace_if_different(ofilename, ofilename_tmp)
 
 def replace_if_different(dst, dst_tmp):
     # If contents are identical, don't touch the file to prevent
@@ -690,31 +741,6 @@ def Popen_safe_legacy(args, write=None, stderr=subprocess.PIPE, **kwargs):
             e = e.decode(errors='replace').replace('\r\n', '\n')
     return p, o, e
 
-def commonpath(paths):
-    '''
-    For use on Python 3.4 where os.path.commonpath is not available.
-    We currently use it everywhere so this receives enough testing.
-    '''
-    # XXX: Replace me with os.path.commonpath when we start requiring Python 3.5
-    import pathlib
-    if not paths:
-        raise ValueError('arg is an empty sequence')
-    common = pathlib.PurePath(paths[0])
-    for path in paths[1:]:
-        new = []
-        path = pathlib.PurePath(path)
-        for c, p in zip(common.parts, path.parts):
-            if c != p:
-                break
-            new.append(c)
-        # Don't convert '' into '.'
-        if not new:
-            common = ''
-            break
-        new = os.path.join(*new)
-        common = pathlib.PurePath(new)
-    return str(common)
-
 def iter_regexin_iter(regexiter, initer):
     '''
     Takes each regular expression in @regexiter and tries to search for it in
@@ -851,7 +877,7 @@ def get_filenames_templates_dict(inputs, outputs):
             values['@INPUT{}@'.format(ii)] = vv
         if len(inputs) == 1:
             # Just one value, substitute @PLAINNAME@ and @BASENAME@
-            values['@PLAINNAME@'] = plain = os.path.split(inputs[0])[1]
+            values['@PLAINNAME@'] = plain = os.path.basename(inputs[0])
             values['@BASENAME@'] = os.path.splitext(plain)[0]
     if outputs:
         # Gather values derived from the outputs, similar to above.
@@ -859,7 +885,7 @@ def get_filenames_templates_dict(inputs, outputs):
         for (ii, vv) in enumerate(outputs):
             values['@OUTPUT{}@'.format(ii)] = vv
         # Outdir should be the same for all outputs
-        values['@OUTDIR@'] = os.path.split(outputs[0])[0]
+        values['@OUTDIR@'] = os.path.dirname(outputs[0])
         # Many external programs fail on empty arguments.
         if values['@OUTDIR@'] == '':
             values['@OUTDIR@'] = '.'
@@ -883,6 +909,31 @@ def windows_proof_rmtree(f):
     # Try one last time and throw if it fails.
     shutil.rmtree(f)
 
+
+def detect_subprojects(spdir_name, current_dir='', result=None):
+    if result is None:
+        result = {}
+    spdir = os.path.join(current_dir, spdir_name)
+    if not os.path.exists(spdir):
+        return result
+    for trial in glob(os.path.join(spdir, '*')):
+        basename = os.path.basename(trial)
+        if trial == 'packagecache':
+            continue
+        append_this = True
+        if os.path.isdir(trial):
+            detect_subprojects(spdir_name, trial, result)
+        elif trial.endswith('.wrap') and os.path.isfile(trial):
+            basename = os.path.splitext(basename)[0]
+        else:
+            append_this = False
+        if append_this:
+            if basename in result:
+                result[basename].append(trial)
+            else:
+                result[basename] = [trial]
+    return result
+
 class OrderedSet(collections.MutableSet):
     """A set that preserves the order in which items are added, by first
     insertion.
